package com.example.topse.shell.commands;

import static org.junit.Assert.*;

import java.io.FileNotFoundException;

import org.jmock.Expectations;
import org.jmock.Mockery;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.example.topse.fs.Directory;
import com.example.topse.fs.DummyFileSystem;
import com.example.topse.fs.File;
import com.example.topse.shell.Shell;

public class CdCommandTest {
	
	CdCommand command = new CdCommand();
	Mockery context = new Mockery();
	Shell shell = context.mock(Shell.class);
	
	DummyFileSystem filesystem = null;

	@Before
	public void setUp() throws Exception {
		filesystem = DummyFileSystem.getInstance();
		filesystem.clear();
	}

	@After
	public void tearDown() throws Exception {
	}
	
	@Test
	public void testDoesNotHandleNotCDCommand() {
		boolean isFinished = false;
		try {
			isFinished = command.execute(shell, "not_cd", null);
		} catch (Exception e) {
			fail();
		}
		assertFalse(isFinished);
	}

	@Test(expected=FileNotFoundException.class)
	public void testFailedToChangeDirectoryIfTheSpecifiedDirectoryDoesNotExit() throws Exception {
		final Directory root = filesystem.getRoot();
		context.checking(new Expectations() { {
			oneOf(shell).getCurrentDirectory(); will(returnValue(root));
		} });
		
		command.execute(shell, "cd", new String[] { "abc" });
	}

	@Test
	public void testMoveToTheChildDirectory() {
		final Directory root = filesystem.getRoot();
		final Directory child = new Directory("abc");
		root.appendChild(child);
		context.checking(new Expectations() { {
			oneOf(shell).getCurrentDirectory(); will(returnValue(root));
			oneOf(shell).setCurrentDirectory(child);
		} });
		boolean isFinished = false;
		try {
			isFinished = command.execute(shell, "cd", new String[] { "abc" });
		} catch (Exception e) {
			fail();
		}
		context.assertIsSatisfied();
		assertTrue(isFinished);
	}

	@Test
	public void testMoveToTheParentDirectory() {
		final Directory root = filesystem.getRoot();
		final Directory child = new Directory("abc");
		root.appendChild(child);
		context.checking(new Expectations() { {
			oneOf(shell).getCurrentDirectory(); will(returnValue(child));
			oneOf(shell).setCurrentDirectory(root);
		} });
		boolean isFinished = false;
		try {
			isFinished = command.execute(shell, "cd", new String[] { ".." });
		} catch (Exception e) {
			fail();
		}
		context.assertIsSatisfied();
		assertTrue(isFinished);
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void testFailedToMoveIfDoNotSupplyValidArguments() throws Exception {
		final Directory root = filesystem.getRoot();
		final Directory child = new Directory("abc");
		root.appendChild(child);
		context.checking(new Expectations() { {
			oneOf(shell).getCurrentDirectory(); will(returnValue(root));
		} });
		boolean isFinished = false;

		isFinished = command.execute(shell, "cd", new String[] { "" });

		context.assertIsSatisfied();
		assertTrue(isFinished);		
	}
	
	@Test(expected = FileNotFoundException.class)
	public void testFailedToMoveIfSpecifyTheFile() throws Exception {
		final Directory root = filesystem.getRoot();
		final File child = new File("abc");
		root.appendChild(child);
		context.checking(new Expectations() { {
			oneOf(shell).getCurrentDirectory(); will(returnValue(root));
		} });

		command.execute(shell, "cd", new String[] { "abc" });
	}
	
	@Test
	public void testMoveToDirectoryAccordingToTheFullPath() {
		final Directory root = filesystem.getRoot();
		final Directory child1 = new Directory("child1");
		final Directory child2 = new Directory("child2");
		final Directory target = new Directory("target");
		root.appendChild(child1);
		root.appendChild(child2);
		child2.appendChild(target);
		context.checking(new Expectations() { {
			oneOf(shell).setCurrentDirectory(target);
		} });
		boolean isFinished = false;
		try {
			isFinished = command.execute(shell, "cd", new String[] { "/child2/target" });
		} catch (Exception e) {
			fail();
		}
		context.assertIsSatisfied();
		assertTrue(isFinished);
	}
	
	@Test
	public void testMoveToDirectoryAccordingToTheComplexRelativePath() {
		final Directory root = filesystem.getRoot();
		final Directory child1 = new Directory("child1");
		final Directory child2 = new Directory("child2");
		final Directory target = new Directory("target");
		root.appendChild(child1);
		root.appendChild(child2);
		child2.appendChild(target);
		context.checking(new Expectations() { {
			oneOf(shell).getCurrentDirectory(); will(returnValue(child1));
			oneOf(shell).setCurrentDirectory(target);
		} });
		boolean isFinished = false;
		try {
			isFinished = command.execute(shell, "cd", new String[] { "../child2/../child1/../child2/target" });
		} catch (Exception e) {
			fail();
		}
		context.assertIsSatisfied();
		assertTrue(isFinished);
	}

	@Test(expected = FileNotFoundException.class)
	public void testFailedToMoveToDirectoryForTheNoExistingTheComplexRelativePath() throws Exception {
		final Directory root = filesystem.getRoot();
		final Directory child1 = new Directory("child1");
		final Directory child2 = new Directory("child2");
		final Directory target = new Directory("target");
		root.appendChild(child1);
		root.appendChild(child2);
		child2.appendChild(target);
		context.checking(new Expectations() { {
			oneOf(shell).getCurrentDirectory(); will(returnValue(child1));
			oneOf(shell).setCurrentDirectory(target);
		} });

		command.execute(shell, "cd", new String[] { "../child2/../child3/../child2/target" });
	}	
	
}
